grammar ExpressionLanguage;

options {
		output = AST;
		language = CSharp2;
		backtrack=true;
		memoize=true;
}

tokens {
		DOT = '.';
		MEMBERACCESS = 'MemberAccess';
		AND = 'And';
		OR = 'Or';
		EQUAL = '=';
		DIFFERENT = '<>';
		LESSTHAN = '<';
		GREATERTHAN = '>';
		LTEQ = '<=';
		GTEQ = '>=';
		LITERALSTRING = 'StringLiteral';
		LITERALNUMERIC = 'NumericLiteral';
		IIF = 'Iif';
		LITERALBOOLEAN = 'BooleanLiteral';
}

@lexer::namespace { TreeNurseryWpf.Grammar }
@parser::namespace { TreeNurseryWpf.Grammar }

conditionalOrExpression
    :    conditionalAndExpression ( OR^ conditionalAndExpression )*;

conditionalAndExpression
    :    equalityExpression ( AND^ equalityExpression )*;

equalityExpression
    :    relationalExpression ( (EQUAL^ | DIFFERENT^) relationalExpression)*;

relationalExpression
    :    atom ( (LESSTHAN^ | LTEQ^ | GREATERTHAN^ | GTEQ^) atom)*;

atom
		: '('! conditionalOrExpression ')'!
		| functionCall
		| memberAccess
		| literal;

expression	
        : conditionalOrExpression 
        | functionCall 
        | parenthesizedExpression 
        | memberAccess 
        | literal;
        
parenthesizedExpression
		:'('! expression ')'!;

memberAccess
        : id=IDENTIFIER 				 	 													-> ^( MEMBERACCESS [$id.text] );      
		
functionCall
		: 'Iif' '(' expression ',' expression ',' expression ')'								-> ^( IIF expression* );

literal
        : sl=STRINGLITERAL																		-> ^( LITERALSTRING [$sl] )
        | bl=BOOLEANLITERAL																		-> ^( LITERALBOOLEAN [$bl] )
        | il=NUMERICLITERAL                                                      				-> ^( LITERALNUMERIC [$il] );

BOOLEANLITERAL
		: 'True'|'False';                  

STRINGLITERAL
        : '\''(~'\'')* '\'';

NUMERICLITERAL
        : (INTEGERLITERAL)* ( DOT ( INTEGERLITERAL ) )?;
         
fragment
INTEGERLITERAL
        : ('0'..'9');
        
IDENTIFIER
        : ( 'a'..'z' | 'A'..'Z' | '_' ) ( 'a'..'z' | 'A'..'Z' | '_' | (INTEGERLITERAL) )* ;

NEWLINE
		: ('\r'? '\n')+			{ $channel = HIDDEN; };

WHITESPACE
        : ( '\t' | ' ' )+     	{ $channel = HIDDEN; } ;
